// simpleFunctions.c

#include <simpleFunctions.h>

void init( void )
{
    ARParam  wparam;

    // open the video path
    if( arVideoOpen( vconf ) < 0 ) exit(0);

    // find the size of the window
    if( arVideoInqSize(&xsize, &ysize) < 0 ) exit(0);
    printf("Image size (x,y) = (%d,%d)\n", xsize, ysize);

    // set the initial camera parameters
    if( arParamLoad(cparam_name, 1, &wparam) < 0 ) {
        printf("Camera parameter load error !!\n");
        exit(0);
    }
    arParamChangeSize( &wparam, xsize, ysize, &cparam );
    arInitCparam( &cparam );
    printf("*** Camera Parameter ***\n");
    arParamDisp( &cparam );

    // load our single pattern file
    if( (patt_id=arLoadPatt(patt_name)) < 0 ) {
        printf("pattern load error !!\n");
        exit(0);
    }

    /* open the graphics window */
    argInit( &cparam, 1.0, 0, 0, 0, 0 );
}


// cleanup function called when program exits
void cleanup(void)
{
    arVideoCapStop();
    arVideoClose();
    argCleanup();
}

// method for handling key presses etc.
void   keyEvent( unsigned char key, int x, int y)
{
    // quit if the ESC key is pressed
    if( key == 0x1b ) {
	// and calculate the average framerate of this program run
        printf("*** %f (frame/sec)\n", (double)count/arUtilTimer());
        cleanup();
        exit(0);
    }
}


// main loop
void mainLoop(void)
{
    ARUint8         *dataPtr;
    ARMarkerInfo    *marker_info;
    int             marker_num;
    int             j, k;

    // grab a video frame
    if( (dataPtr = (ARUint8 *)arVideoGetImage()) == NULL ) {
        arUtilSleep(2);
        return;
    }
    if( count == 0 ) arUtilTimerReset();
    count++;

    // display the frame as background
    argDrawMode2D();
    argDispImage( dataPtr, 0,0 );

    // detect the markers in the video frame
    if( arDetectMarker(dataPtr, thresh, &marker_info, &marker_num) < 0 ) {
        cleanup();
        exit(0);
    }

    // trigger capture of next video frame (in the background)
    arVideoCapNext();

    // check for correct marker visibility
    k = -1;
    for( j = 0; j < marker_num; j++ ) {
        if( patt_id == marker_info[j].id ) {
            if( k == -1 ) k = j;
            else if( marker_info[k].cf < marker_info[j].cf ) k = j;
        }
    }
    if( k == -1 ) {
        // no marker found...
        argSwapBuffers();
        return;
    }

    // get the transformation between the marker and the camera
    arGetTransMat(&marker_info[k], patt_center, patt_width, patt_trans);

    // use this info to draw augmentations in a subroutine
    draw();

    // double buffering is looking good...
    argSwapBuffers();

    // start again
}


void draw( void )
{
    // these variables influence how things are drawn, 
    // see below what they do and play around with them!
    double    gl_para[16];
    GLfloat   mat_ambient[]     = {0.0, 0.0, 1.0, 1.0};
    GLfloat   mat_flash[]       = {0.0, 0.0, 1.0, 1.0};
    GLfloat   mat_flash_shiny[] = {50.0};
    GLfloat   light_position[]  = {100.0,-200.0,200.0,0.0};
    GLfloat   ambi[]            = {0.1, 0.1, 0.1, 0.1};
    GLfloat   lightZeroColor[]  = {0.9, 0.9, 0.9, 0.1};
   
    // initialization operations
    argDrawMode3D();
    argDraw3dCamera( 0, 0 );
    glClearDepth( 1.0 );
    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    
    // load the camera transformation matrix
    // i.e. set the OpenGL camera such that it is at the
    // same pose as our real camera 
    argConvGlpara(patt_trans, gl_para);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixd( gl_para );

    // OpenGL stuff: set up light source... 
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambi);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor);
    // select a material for rendered object...
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_flash);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_flash_shiny);	
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    // transform the next object to be rendered
    glMatrixMode(GL_MODELVIEW);
    // 25 mm along the z-axis
    glTranslatef( 0.0, 0.0, 25.0 );
    // now render the object: a boring cube of 50mm diameter
    glutSolidCube(50.0);

    // cleanup operations necessary for correct drawing of video
    // background in next video frame
    glDisable( GL_LIGHTING );
    glDisable( GL_DEPTH_TEST );
}
